// ALWAYS INCLUDE <click/config.h> FIRST
#include <click/config.h>
// include your own config.h if appropriate
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "broccolielt.hh"
#include <click/error.hh>

BroccoliElement::BroccoliElement()
  : Element(1,1), _bc(0)
{
  // CONSTRUCTOR MUST MOD_INC_USE_COUNT
  MOD_INC_USE_COUNT;
}

BroccoliElement::~BroccoliElement()
{
  bro_disconnect(_bc);

  // DESTRUCTOR MUST MOD_DEC_USE_COUNT
  MOD_DEC_USE_COUNT;
}

int
BroccoliElement::initialize(ErrorHandler *errh)
{
  const char *bro_host;
  
  bro_conf_set_domain("click");
  if (! (bro_host = bro_conf_get_str("BroHost")))
    return -1;
  
  // Try connect, if it fails, we continue regardless.
  if ( ! (_bc = bro_connect_str(bro_host, BRO_CFLAG_COMPLETE_HANDSHAKE)))
    errh->warning("Could not connect to Bro");

  return 0;
}

static String
read_handler(Element *, void *)
{
  return "false\n";
}

void
BroccoliElement::add_handlers()
{
  // needed for QuitWatcher
  add_read_handler("scheduled", read_handler, 0);
}

void
BroccoliElement::push(int port, Packet *packet)
{
  BroEvent *ev;
  // Get some useful information out of the packet and ship an event to Bro:
  const click_ip* iphdr = packet->ip_header();

  ev = bro_event_new("click_packet_processed");
  bro_event_add_val(ev, BRO_TYPE_IPADDR, &(iphdr->ip_src));
  bro_event_add_val(ev, BRO_TYPE_IPADDR, &(iphdr->ip_dst));
  bro_event_send(_bc, ev);
  bro_event_free(ev);

  Element::push(port, packet);
}

EXPORT_ELEMENT(BroccoliElement)
